package com.flyfish.framework.approval.controller;

import com.flyfish.framework.approval.domain.ApprovalDomain;
import com.flyfish.framework.approval.domain.record.ApproveRecord;
import com.flyfish.framework.approval.domain.record.ApproveRecordListVo;
import com.flyfish.framework.approval.domain.record.ApproveRecordQo;
import com.flyfish.framework.approval.domain.todo.ApprovalListVo;
import com.flyfish.framework.approval.service.ModuleDelegateService;
import com.flyfish.framework.bean.Result;
import com.flyfish.framework.configuration.annotations.CurrentUser;
import com.flyfish.framework.configuration.annotations.PagedQuery;
import com.flyfish.framework.controller.reactive.ReactiveBaseController;
import com.flyfish.framework.domain.base.Vo;
import com.flyfish.framework.domain.po.User;
import org.springframework.web.bind.annotation.*;
import reactor.core.publisher.Mono;

import javax.annotation.Resource;
import java.util.List;

/**
 * 审批记录控制器
 *
 * @author wangyu
 */
@RestController
@RequestMapping("approves")
public class ApproveRecordController extends ReactiveBaseController<ApproveRecord, ApproveRecordQo> {

    @Resource
    private ModuleDelegateService moduleDelegateService;

    /**
     * 查询视图
     *
     * @return 结果
     */
    @GetMapping("records")
    public Mono<Result<List<Vo<ApproveRecord>>>> views(@PagedQuery ApproveRecordQo qo) {
        return reactiveService.getPageList(qo)
                .map(Result::accept)
                .map(result -> result.mapList((ApproveRecord item) -> new ApproveRecordListVo().from(item)));
    }

    /**
     * 查询待办列表
     *
     * @param qo   查询实体
     * @param user 用户
     * @return 结果
     */
    @GetMapping("todos")
    public Mono<Result<List<Vo<ApproveRecord>>>> todos(@PagedQuery ApproveRecordQo qo, @CurrentUser User user) {
        qo.setApprover(user.getId());
        qo.setApproved(false);
        return reactiveService.getPageList(qo)
                .map(Result::accept)
                .map(result -> result.mapList((ApproveRecord item) -> new ApprovalListVo().from(item)));
    }

    /**
     * 通用审批
     *
     * @param module 模块
     * @param dataId 数据id
     * @return 结果
     */
    @PostMapping("approval/{module}/{dataId}")
    public Mono<Result<ApprovalDomain>> approval(@PathVariable String module, @PathVariable String dataId,
                                                 @RequestBody ApproveRecord record) {
        record.setModule(module);
        record.setDataId(dataId);
        return moduleDelegateService.approval(record).map(Result::ok);
    }
}
